//
// Copyright (c) 2012, MindFusion LLC - Bulgaria.
//

using System;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;

using MindFusion.Diagramming.Commands;
using MindFusion.Drawing;

using Pen = System.Drawing.Pen;
using SolidBrush = MindFusion.Drawing.SolidBrush;


namespace MindFusion.Diagramming.WinForms.Samples.CS.FlowCharter
{
	/// <summary>
	/// Summary description for MainForm.
	/// </summary>
	public partial class MainForm : Form
	{
		private class NodeProps
		{
			[Category("Properties")]
			[Description("The text displayed within the node.")]
			public string Text
			{
				get { return _text; }
				set { _text = value; }
			}

			[Category("Properties")]
			[Description("The font of the node's text.")]
			public Font Font
			{
				get { return _font; }
				set { _font = value; }
			}

			[Category("Properties")]
			[Description("The interior color of the node.")]
			public Color FillColor
			{
				get { return _fillColor; }
				set { _fillColor = value; }
			}

			private string _text;
			private Font _font;
			private Color _fillColor;
		}

		private class NodeDragItem
		{
			public NodeDragItem(int index)
			{
				_index = index;
			}

			public static Type _GetType()
			{
				return new NodeDragItem(0).GetType();
			}

			public int Index
			{
				get { return _index; }
			}

			private int _index;
		}

		private class Node
		{
			public Node(AnchorPattern anchor,
				Shape template, string name)
			{
				_anchor = anchor;
				_template = template;
				_name = name;
			}

			public AnchorPattern Anchor
			{
				get { return _anchor; }
			}

			public Shape Template
			{
				get { return _template; }
			}

			public string Name
			{
				get { return _name; }
			}

			private AnchorPattern _anchor;
			private Shape _template;
			private string _name;
		}

		private class Connector
		{
			public Connector(Shape head, string name)
			{
				_head = head;
				_name = name;
			}

			public Shape Head
			{
				get { return _head; }
			}

			public string Name
			{
				get { return _name; }
			}

			private Shape _head;
			private string _name;
		}

		public MainForm()
		{
			InitializeComponent();

            diagram.BackBrush = new SolidBrush(Color.FromArgb(224, 233, 233));
            diagram.ShapeNodeStyle.Brush = new SolidBrush(Color.FromArgb(156, 170, 198));
            diagram.ShapeNodeStyle.Stroke = new SolidBrush(Color.FromArgb(0, 0, 99));
            diagram.DiagramLinkStyle.Brush = new SolidBrush(Color.FromArgb(102, 154, 204));
            diagram.DiagramLinkStyle.Stroke = new SolidBrush(Color.FromArgb(0, 0, 99));

#if !STANDARD
			diagram.UndoManager.UndoEnabled = true;
			diagram.UndoManager.History.Capacity = 30;
#endif

			Color defAnch = Color.Red;

			_nodes = new ShapeNode[]
				{
					new ShapeNode {
						AnchorPattern = new AnchorPattern(new AnchorPoint[]
						{
							new AnchorPoint(50, 0, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(0, 100, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(50, 100, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(100, 100, true, true, MarkStyle.Circle, defAnch)
						}),
						Shape = Shape.FromId("Alternative"), // get shape by string id
						Tag = "Alternative"
					},
					new ShapeNode {
						AnchorPattern = new AnchorPattern(new AnchorPoint[]
						{
							new AnchorPoint(30, 0, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(50, 0, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(100, 0, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(100, 50, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(100, 100, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(50, 100, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(0, 100, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(0, 50, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(0, 30, true, true, MarkStyle.Circle, defAnch)
						}),
						Shape = Shape.FromId("PunchedCard"),
						Tag = "Card"
					},
					new ShapeNode {
						AnchorPattern = AnchorPattern.Decision2In2Out,
						Shape = Shape.FromId("Decision"),
						Tag = "Decision"
					},
					new ShapeNode {
						AnchorPattern = new AnchorPattern(new AnchorPoint[]
						{
							new AnchorPoint(0, 0, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(50, 0, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(100, 50, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(50, 100, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(0, 100, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(0, 50, true, true, MarkStyle.Circle, defAnch)
						}),
						Shape = Shape.FromId("Delay"),
						Tag = "Delay"
					},
					new ShapeNode {
						AnchorPattern = new AnchorPattern(new AnchorPoint[]
						{
							new AnchorPoint(15, 0, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(50, 0, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(85, 0, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(100, 50, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(85, 100, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(50, 100, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(15, 100, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(0, 50, true, true, MarkStyle.Circle, defAnch)
						}),
						Shape = Shapes.DirectAccessStorage,
						Tag = "Direct data"
					},
					new ShapeNode {
						AnchorPattern = new AnchorPattern(new AnchorPoint[]
						{
							new AnchorPoint(50, 0, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(100, 20, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(100, 50, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(100, 80, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(50, 100, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(0, 80, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(0, 50, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(0, 20, true, true, MarkStyle.Circle, defAnch)
						}),
						Shape = Shape.FromId("DiskStorage"),
						Tag = "DiskStorage"
					},
					new ShapeNode {
						AnchorPattern = new AnchorPattern(new AnchorPoint[]
						{
							new AnchorPoint(40, 0, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(85, 0, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(100, 50, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(85, 100, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(40, 100, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(0, 50, true, true, MarkStyle.Circle, defAnch)
						}),
						Shape = Shape.FromId("Display"),
						Tag = "Display"
					},
					new ShapeNode {
						AnchorPattern = new AnchorPattern(new AnchorPoint[]
						{
							new AnchorPoint(50, 0, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(100, 50, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(50, 100, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(0, 50, true, true, MarkStyle.Circle, defAnch)
						}),
						Shape = Shape.FromId("DividedEvent"),
						Tag = "Divided event"
					},
					new ShapeNode {
						AnchorPattern = new AnchorPattern(new AnchorPoint[]
						{
							new AnchorPoint(0, 0, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(50, 0, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(100, 0, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(100, 50, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(100, 100, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(50, 100, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(0, 100, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(0, 50, true, true, MarkStyle.Circle, defAnch)
						}),
						Shape = Shape.FromId("DividedProcess"),
						Tag = "Divided process"
					},
					new ShapeNode {
						AnchorPattern = new AnchorPattern(new AnchorPoint[]
						{
							new AnchorPoint(0, 0, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(50, 0, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(100, 0, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(100, 50, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(100, 90, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(50, 90, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(0, 90, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(0, 50, true, true, MarkStyle.Circle, defAnch)
						}),
						Shape = Shape.FromId("Document"),
						Tag = "Document"
					},
					new ShapeNode {
						AnchorPattern = new AnchorPattern(new AnchorPoint[]
						{
							new AnchorPoint(0, 0, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(50, 0, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(100, 0, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(100, 50, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(100, 100, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(50, 100, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(0, 100, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(0, 50, true, true, MarkStyle.Circle, defAnch)
						}),
						Shape = Shape.FromId("InternalStorage"),
						Tag = "Internal storage"
					},
					new ShapeNode {
						AnchorPattern = new AnchorPattern(new AnchorPoint[]
						{
							new AnchorPoint(25, 0, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(50, 0, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(75, 0, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(100, 25, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(100, 50, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(100, 100, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(50, 100, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(0, 100, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(0, 50, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(0, 25, true, true, MarkStyle.Circle, defAnch)
						}),
						Shape = Shape.FromId("BeginLoop"),
						Tag = "Loop limit"
					},
					new ShapeNode {
						AnchorPattern = new AnchorPattern(new AnchorPoint[]
						{
							new AnchorPoint(50, 20, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(100, 0, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(100, 50, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(100, 100, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(50, 100, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(0, 100, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(0, 50, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(0, 40, true, true, MarkStyle.Circle, defAnch)
						}),
						Shape = Shape.FromId("Input"),
						Tag = "Manual input"
					},
					new ShapeNode {
						AnchorPattern = new AnchorPattern(new AnchorPoint[]
						{
							new AnchorPoint(0, 0, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(50, 0, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(100, 0, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(80, 100, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(50, 100, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(20, 100, true, true, MarkStyle.Circle, defAnch)
						}),
						Shape = Shape.FromId("ManualOperation"),
						Tag = "Manual operation"
					},
					new ShapeNode {
						AnchorPattern = new AnchorPattern(new AnchorPoint[]
						{
							new AnchorPoint(0, 0, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(50, 0, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(100, 0, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(80, 50, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(100, 100, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(50, 100, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(0, 100, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(0, 50, true, true, MarkStyle.Circle, defAnch)
						}),
						Shape = Shape.FromId("MessageFromUser"),
						Tag = "Message from User"
					},
					new ShapeNode {
						AnchorPattern = new AnchorPattern(new AnchorPoint[]
						{
							new AnchorPoint(20, 0, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(50, 0, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(100, 0, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(100, 50, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(100, 100, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(50, 100, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(20, 100, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(0, 50, true, true, MarkStyle.Circle, defAnch)
						}),
						Shape = Shape.FromId("MessageToUser"),
						Tag = "Message to User"
					},
					new ShapeNode {
						AnchorPattern = new AnchorPattern(new AnchorPoint[]
						{
							new AnchorPoint(50, 0, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(100, 0, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(100, 50, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(45, 90, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(0, 90, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(0, 50, true, true, MarkStyle.Circle, defAnch)
						}),
						Shape = Shape.FromId("MultiDocument"),
						Tag = "Multi-document"
					},
					new ShapeNode {
						AnchorPattern = new AnchorPattern(new AnchorPoint[]
						{
							new AnchorPoint(0, 10, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(50, 10, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(100, 10, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(100, 50, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(100, 90, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(50, 90, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(0, 90, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(0, 50, true, true, MarkStyle.Circle, defAnch)
						}),
						Shape = Shape.FromId("Microform"),
						Tag = "Paper tape"
					},
					new ShapeNode {
						AnchorPattern = new AnchorPattern(new AnchorPoint[]
						{
							new AnchorPoint(0, 0, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(50, 0, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(100, 0, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(80, 50, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(100, 100, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(50, 100, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(0, 100, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(0, 50, true, true, MarkStyle.Circle, defAnch)
						}),
						Shape = Shape.FromId("PrimitiveFromCall"),
						Tag = "Primitive from Call"
					},
					new ShapeNode {
						AnchorPattern = new AnchorPattern(new AnchorPoint[]
						{
							new AnchorPoint(20, 0, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(50, 0, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(100, 0, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(100, 50, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(100, 100, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(50, 100, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(20, 100, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(0, 50, true, true, MarkStyle.Circle, defAnch)
						}),
						Shape = Shape.FromId("PrimitiveToCall"),
						Tag = "Primitive to Call"
					},
					new ShapeNode {
						AnchorPattern = new AnchorPattern(new AnchorPoint[]
						{
							new AnchorPoint(0, 0, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(50, 0, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(100, 0, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(100, 50, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(100, 100, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(50, 100, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(0, 100, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(0, 50, true, true, MarkStyle.Circle, defAnch)
						}),
						Shape = Shape.FromId("Procedure"),
						Tag = "Procedure"
					},
					new ShapeNode {
						AnchorPattern = new AnchorPattern(new AnchorPoint[]
						{
							new AnchorPoint(15, 0, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(50, 0, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(85, 0, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(100, 30, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(100, 50, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(100, 70, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(15, 100, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(50, 100, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(85, 100, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(0, 30, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(0, 50, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(0, 70, true, true, MarkStyle.Circle, defAnch)
						}),
						Shape = Shape.FromId("Start"),
						Tag = "Start"
					},
					new ShapeNode {
						AnchorPattern = new AnchorPattern(new AnchorPoint[]
						{
							new AnchorPoint(50, 0, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(0, 50, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(50, 100, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(100, 50, true, true, MarkStyle.Circle, defAnch),
							new AnchorPoint(100, 100, true, true, MarkStyle.Circle, defAnch)
						}),
						Shape = Shape.FromId("Tape"),
						Tag = "Tape"
					}
				};
			foreach (ShapeNode s in _nodes)
				_nodeList.AddNode(s, s.Tag.ToString());

			_connectors = new Connector[]
				{
					new Connector(
						ArrowHeads.Arrow,
						"Arrow"),
					new Connector(
						ArrowHeads.BackSlash,
						"Back slash"),
					new Connector(
						ArrowHeads.BowArrow,
						"Bow arrow"),
					new Connector(
						ArrowHeads.Circle,
						"Circle"),
					new Connector(
						ArrowHeads.DoubleArrow,
						"Double arrow"),
					new Connector(
						ArrowHeads.None,
						"None"),
					new Connector(
						ArrowHeads.Pentagon,
						"Pentagon"),
					new Connector(
						ArrowHeads.PointerArrow,
						"Pointer"),
					new Connector(
						ArrowHeads.Quill,
						"Quill"),
					new Connector(
						ArrowHeads.Reversed,
						"Reversed"),
					new Connector(
						ArrowHeads.RevTriangle,
						"Reversed triangle"),
					new Connector(
						ArrowHeads.RevWithCirc,
						"Reversed with Circle"),
					new Connector(
						ArrowHeads.RevWithLine,
						"Reversed with Line"),
					new Connector(
						ArrowHeads.Rhombus,
						"Rhombus"),
					new Connector(
						ArrowHeads.Slash,
						"Slash"),
					new Connector(
						ArrowHeads.Tetragon,
						"Tetragon"),
					new Connector(
						ArrowHeads.Triangle,
						"Triangle")
				};
			foreach (Connector c in _connectors)
				_connectorList.Items.Add(c.Name);

			_connectorList.SelectedIndex = 0;
			_connectorTypeCombo.SelectedIndex = 0;

			// Set-up the hidden chart
			_hidden.Bounds = new RectangleF(0, 0, 50, 50);
			_hidden.RestrictItemsToBounds = RestrictToBounds.NoRestriction;

			// Set-up the flow chart
			diagram.Selection.AllowMultipleSelection = false;
			diagram.LinkCascadeOrientation = Orientation.Auto;
		}

		private void ListBox_DrawItem(ListBox lb, System.Windows.Forms.DrawItemEventArgs e)
		{
			Rectangle rcc;
			RectangleF rc;

			System.Drawing.SolidBrush brush;
			Pen selPen;
			System.Drawing.SolidBrush selBrush;
			StringFormat sf;

			rcc = e.Bounds;
			rcc.Inflate(-1, -1);
			rc = new RectangleF((float)e.Bounds.Left + 32,
				e.Bounds.Top, (float)e.Bounds.Width - 32,
				e.Bounds.Height);

			brush = new System.Drawing.SolidBrush(Color.Black);
			selPen = new Pen(Color.Goldenrod);
			selBrush = new System.Drawing.SolidBrush(Color.LightYellow);

			sf = StringFormat.GenericDefault;
			sf.LineAlignment = StringAlignment.Center;

			// Draw the selection if any
			if ((e.State & DrawItemState.Selected) > 0)
			{
				e.Graphics.FillRectangle(selBrush, rcc);
				e.Graphics.DrawRectangle(selPen, rcc);
			}
			else
			{
				System.Drawing.SolidBrush tb = new
					System.Drawing.SolidBrush(Color.White);
				e.Graphics.FillRectangle(tb, e.Bounds);
				tb.Dispose();
			}

			e.Graphics.DrawString((string)lb.Items[e.Index],
				_nodeList.Font, brush, rc, sf);

			brush.Dispose();
			selPen.Dispose();
			selBrush.Dispose();
		}


		private void NodeList_OnMeasureItem(object sender, MeasureItemEventArgs e)
		{
			e.ItemHeight = 32;
		}

		private void ConnectorList_OnMeasureItem(object sender, MeasureItemEventArgs e)
		{
			e.ItemHeight = 32;
		}

		private void ConnectorList_OnDrawItem(object sender, System.Windows.Forms.DrawItemEventArgs e)
		{
			if (e.Index < 0)
				return;

			// Draw the item
			ListBox_DrawItem(_connectorList, e);

			Pen pen = new Pen(Color.Black);
			Point p0 = new Point(e.Bounds.X + 2, e.Bounds.Y + 13);
			Point p1 = new Point(e.Bounds.X + 19, e.Bounds.Y + 13);

			e.Graphics.SmoothingMode = SmoothingMode.AntiAlias;
			e.Graphics.DrawLine(Pens.Black, p0, p1);

			DiagramLink.DrawArrowhead(new GdiGraphics(e.Graphics), pen, Brushes.LightSkyBlue,
				_connectors[e.Index].Head, new PointF(50, 0), p1, p0, 12F, false);

			pen.Dispose();
		}

		private void ConnectorList_OnSelectionChanged(object sender, EventArgs e)
		{
			int si = _connectorList.SelectedIndex;
			if (si < 0 || si >= _connectors.Length)
				return;

			diagram.LinkHeadShape = _connectors[si].Head;
		}

		private void OnPropertyValueChanged(object s, PropertyValueChangedEventArgs e)
		{
			if (diagram.Selection.Nodes.Count == 0)
				return;

			ShapeNode b = diagram.Selection.Nodes[0] as ShapeNode;
			if (b == null)
				return;

			// Create undo record
			ChangeItemCmd cmd =
				new ChangeItemCmd(b, "Property change");

			b.Text = _selected.Text;
			b.Font = _selected.Font;
			b.Brush = new SolidBrush(_selected.FillColor);

			cmd.Execute();
		}

		private void OnFileNew(object sender, EventArgs e)
		{
			diagram.ClearAll();
		}

		private void OnFileOpen(object sender, EventArgs e)
		{
			if (_openFileDialog.ShowDialog(this) == DialogResult.OK)
			{
				try
				{
					diagram.LoadFromFile(_openFileDialog.FileName);
				}
				catch
				{
					MessageBox.Show(this, "Failed to open document!");
				}
			}
		}

		private void OnFileSave(object sender, EventArgs e)
		{
			if (_saveFileDialog.ShowDialog(this) == DialogResult.OK)
			{
				diagram.SaveToFile(_saveFileDialog.FileName, true);
			}
		}

		private void OnPrintPreview(object sender, EventArgs e)
		{
			diagramView.PrintOptions.DocumentName = "Flowchart";
			diagramView.PrintOptions.EnableImages = false;
			diagramView.PrintOptions.EnableInterior = true;
			diagramView.PrintOptions.EnableShadows = true;
			diagramView.PrintOptions.Scale = 100;
			diagramView.PrintPreview();
		}

		private void OnPrint(object sender, EventArgs e)
		{
			diagramView.PrintOptions.DocumentName = "Flowchart";
			diagramView.PrintOptions.EnableImages = false;
			diagramView.PrintOptions.EnableInterior = true;
			diagramView.PrintOptions.EnableShadows = true;
			diagramView.PrintOptions.Scale = 100;
			diagramView.Print();
		}

		private void OnExit(object sender, EventArgs e)
		{
			Close();
		}

		private void OnOptions(object sender, EventArgs e)
		{
			OptionsDialog dlg = new OptionsDialog();

			dlg.ShowGrid = diagram.ShowGrid;
			dlg.BackgroundColor = (diagram.BackBrush as SolidBrush).Color;
			dlg.NodeColor = (diagram.ShapeNodeStyle.Brush as SolidBrush).Color;
			dlg.ConnectorColor = (diagram.DiagramLinkStyle.Brush as SolidBrush).Color;
			dlg.AnchorColor = _anchorColor;

			dlg.ShowAnchors = diagram.ShowAnchors != ShowAnchors.Never;
			switch (_anchorStyle)
			{
			case MarkStyle.Circle:
				dlg.AnchorStyle = 0;
				break;
			case MarkStyle.Rectangle:
				dlg.AnchorStyle = 1;
				break;
			case MarkStyle.X:
				dlg.AnchorStyle = 2;
				break;
			default:
				dlg.AnchorStyle = 0;
				break;
			}

			if (dlg.ShowDialog(this) == DialogResult.OK)
			{
				diagram.ShowGrid = dlg.ShowGrid;
				diagram.BackBrush = new SolidBrush(dlg.BackgroundColor);
				diagram.ShapeNodeStyle.Brush = new SolidBrush(dlg.NodeColor);
				diagram.DiagramLinkStyle.Brush = new SolidBrush(dlg.ConnectorColor);
				_anchorColor = dlg.AnchorColor;

				diagram.ShowAnchors = dlg.ShowAnchors ?
					ShowAnchors.Auto : ShowAnchors.Never;
				switch (dlg.AnchorStyle)
				{
				case 0:
					_anchorStyle = MarkStyle.Circle;
					break;
				case 1:
					_anchorStyle = MarkStyle.Rectangle;
					break;
				case 2:
					_anchorStyle = MarkStyle.X;
					break;
				default:
					_anchorStyle = MarkStyle.Circle;
					break;
				}
			}
		}

		private void OnAbout(object sender, EventArgs e)
		{
			AboutDialog dlg = new AboutDialog();

			dlg.ShowDialog(this);
		}

		private void ConnectorTypeCombo_OnSelectedValueChange(object sender, EventArgs e)
		{
			LinkShape shape = LinkShape.Polyline;
			short segments = 1;

			switch (_connectorTypeCombo.SelectedIndex)
			{
			case 0:
				shape = LinkShape.Polyline;
				break;
			case 1:
				shape = LinkShape.Bezier;
				break;
			case 2:
				shape = LinkShape.Cascading;
				segments = 3;
				break;
			}

			diagram.LinkShape = shape;
			diagram.LinkSegments = segments;
		}

		private void OnDelete(object sender, EventArgs e)
		{
			if (diagram.Selection.Nodes.Count > 0)
				diagram.Nodes.Remove(diagram.Selection.Nodes[0]);
			if (diagram.Selection.Links.Count > 0)
				diagram.Links.Remove(diagram.Selection.Links[0]);
		}

		private void OnEditPopup(object sender, EventArgs e)
		{
#if !STANDARD
			if (diagram.UndoManager.History.NextUndo != null)
			{
				_menuUndo.Enabled = true;
				_menuUndo.Text = "Undo " + diagram.UndoManager.History.NextUndo.Title;
			}
			else
			{
				_menuUndo.Enabled = false;
				_menuUndo.Text = "Undo";
			}
			if (diagram.UndoManager.History.NextRedo != null)
			{
				_menuRedo.Enabled = true;
				_menuRedo.Text = "Redo " + diagram.UndoManager.History.NextRedo.Title;
			}
			else
			{
				_menuRedo.Enabled = false;
				_menuRedo.Text = "Redo";
			}
#endif
		}

		private void OnUndo(object sender, EventArgs e)
		{
#if !STANDARD
			diagram.UndoManager.Undo();
#endif
		}

		private void OnRedo(object sender, EventArgs e)
		{
#if !STANDARD
			diagram.UndoManager.Redo();
#endif
		}

		private void diagram_NodeDeactivated(object sender, NodeEventArgs e)
		{
			_propertyGrid.SelectedObject = null;
			_selected = null;
		}

		private void diagram_NodeActivated(object sender, NodeEventArgs e)
		{
			ShapeNode node = e.Node as ShapeNode;
			if (node != null)
			{
				_selected = new NodeProps();
				_selected.Text = node.Text;
				_selected.Font = node.EffectiveFont;
				_selected.FillColor = (node.EffectiveBrush as SolidBrush).Color;

				_propertyGrid.SelectedObject = _selected;
			}
		}

		private void diagram_NodeClicked(object sender, NodeEventArgs e)
		{
			if (e.MouseButton != MouseButton.Right)
				return;

			Point pt = diagramView.DocToClient(e.MousePosition);
			_contextMenu.Show(diagramView, pt);
		}

		private void diagram_LinkClicked(object sender, LinkEventArgs e)
		{
			if (e.MouseButton != MouseButton.Right)
				return;

			Point pt = diagramView.DocToClient(e.MousePosition);
			_contextMenu.Show(diagramView, pt);
		}


		// Data
		private static ShapeNode[] _nodes = null;
		private static Connector[] _connectors = null;

		private NodeProps _selected = null;

		private Color _anchorColor = Color.Red;
		private MarkStyle _anchorStyle = MarkStyle.Circle;

		private void diagramView_DragDrop(object sender, DragEventArgs e)
		{

		}

		private void diagramView_DragOver(object sender, DragEventArgs e)
		{

		}
	}
}